//------------------------------------------------------------------------
// Project     : VST SDK
//
// Category    : Examples
// Filename    : public.sdk/samples/vst/pitchnames/source/version.h
// Created by  : Steinberg, 12/2010
// Description : Example of handle the versioning and copyright info of pitchnames plug-in 
//				 used for the resources (RC file for example)
//
//-----------------------------------------------------------------------------
// This file is part of a Steinberg SDK. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this distribution
// and at www.steinberg.net/sdklicenses.
// No part of the SDK, including this file, may be copied, modified, propagated,
// or distributed except according to the terms contained in the LICENSE file.
//-----------------------------------------------------------------------------

#pragma once

#include "pluginterfaces/base/fplatform.h"

// Plain project version file generated by cmake
#include "projectversion.h"

#define stringOriginalFilename	"PitchNames.vst3"
#if SMTG_PLATFORM_64
#define stringFileDescription	"PitchNames VST3-SDK (64Bit)"
#else
#define stringFileDescription	"PitchNames VST3-SDK"
#endif
#define stringCompanyWeb		"http://www.steinberg.net"
#define stringCompanyEmail		"mailto:info@steinberg.de"
#define stringCompanyName		"Steinberg Media Technologies"
#define stringLegalCopyright	"© 2025 Steinberg Media Technologies"
#define stringLegalTrademarks	"VST is a trademark of Steinberg Media Technologies GmbH"
