//------------------------------------------------------------------------
// Project     : VST SDK
//
// Category    : Examples
// Filename    : public.sdk/samples/vst/prefetchablesupport/source/plugcids.h
// Created by  : Steinberg, 04/2015
// Description : define the class IDs for prefetchablesupport
//
//-----------------------------------------------------------------------------
// This file is part of a Steinberg SDK. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this distribution
// and at www.steinberg.net/sdklicenses.
// No part of the SDK, including this file, may be copied, modified, propagated,
// or distributed except according to the terms contained in the LICENSE file.
//-----------------------------------------------------------------------------

#pragma once

namespace Steinberg {
namespace Vst {
static const FUID PlugProcessorUID (0xABFECE0, 0x1A93E746, 0x8341C634, 0x2394798D);
static const FUID PlugControllerUID (0xC2472691, 0xB1FA6441, 0xB272196A, 0xD5CE58BB);

}} // namespaces
